/*
 Name:		Arduino-IC-Tester.ino
 Created:	18.12.2022
 Author:	   Ulli


 2022-012-18 Version 1
 -----------------------
 Basis-Version
*/


#include <UrsArduinoSoftI2cEx.h>
#include <Wire.h>
#include "Packet.h"
#include "TestIc.h"
#include "PcPort.h"
#include "Config.h"

uint8_t rxPin = 11;
uint8_t txPin = 12;

TestIcClass testIC;

#ifdef DEBUG
SoftwareSerial SeriallLogger(SoftUartRX, SoftUartTX);
#endif // DEBUG



void setup() {
   PcPort.init();
   Wire.begin();
   Wire.setClock(100000);
   delay(100);

#ifdef DEBUG
   SeriallLogger.begin(SoftUartBaud);
   DebugPrintln("Is running");
#endif // DEBUG


   // Bereitschaft anzeigen
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);
}

void loop() {
   // Auf das nchste Paket warten.
   if (PcPort.readPacket() < 0) {
      PcPort.sendBadCrc();
      return;
   }


   // Es gibt ein neues Paket.
   DebugVar("Packet: ", packet.type);


   // Pakettyp: Start
   if (packet.type == 0) {
      testIC.initTest();
      PcPort.sendOK();
      return;
   }

   // Pakettyp: Pinmode
   if (packet.type == 1) {
      if (testIC.setPinModes()) {
         PcPort.sendOK();
      }
      else {
         PcPort.sendBadPinType();
      }
      return;
   }

   // Pakettyp: einfacher Testschritt
   if (packet.type == 2) {
      // GPIO-Werte setzen (<- packet)
      testIC.setGpioData();
      // GPIO-Werte einlesen (-> packet)
      delay(100); // Ohne dieses delay gab es bertragungsprobleme. Ursache unbekannt.
      testIC.buildGpioDataPacket();
      // Eingelesene Daten (packet) bertragen
      PcPort.sendResultPacket();
      return;
   }

   // Pakettyp: Testschritt mit I2C-bertragung
   if (packet.type == 3) {
      if (testIC.sda == 0xFF || testIC.scl == 0xFF) {
         PcPort.sendBadI2CPins();
         return;
      }

      // GPIO-Werte setzen (<- packet)
      // -----------------
      testIC.setGpioData();

      // I2C-Datenpakete an Test-IC senden
      // ----------------------------------
      // index auf Index des Bytes mit der Anzahl der Xmit-Pakete
      uint8_t index = testIC.numberOfPins;
      uint8_t packetCount = packet.data[index++];
      int rtc = SoftI2cEx.begin(softSDA, softSCL);
      if (rtc != 0) { // Fehler
         PcPort.sendi2cBeginError();
         return;
      }

      // ber alle Pakete
      // index zeigt auf das erste Byte des Pakets = Anzahl Bytes inkl. Slave-Adresse
      for (uint8_t i = 0; i < packetCount; i++) {
         uint8_t byteCount = packet.data[index++];
         // index zeigt auf das erste zu versendende Byte des Pakets == Adresse des Slaves

         DebugVarHex("Addr: 0x", packet.data[index] << 1);
#ifdef DEBUG
         uint8_t* p = &(packet.data[index + 1]);
         for (uint8_t i = 0; i < byteCount - 1; i++) {
            DebugPrintHex(*p++); DebugPrint(' ');
         }
         DebugPrintln();
#endif // DEBUG


         rtc = SoftI2cEx.writePacket(packet.data[index], &(packet.data[index + 1]), byteCount - 1);

         if (rtc == UrsArduinoSoftI2cExClass::ErrorCode::NoSlave) {
            PcPort.sendI2cInvalidAdress();
            return;
         }
         if (rtc != byteCount - 1) {
            PcPort.sendI2cIoError();
            return;
         }
         index += byteCount;
         // index auf Anzahl der Bytes des Folgepakets bzw. Gerteadresse zum Einlesen
      }

      // GPIO-Werte einlesen (-> packet)
      delay(100);
      testIC.buildGpioDataPacket();
      // Eingelesene Daten (packet) bertragen
      PcPort.sendResultPacket();
      return;
   } // Ende (packet.type == 3)

// Pakettyp: Testschritt mit I2C-bertragung und -Einlesen
   if (packet.type == 4) {
      if (testIC.sda == 0xFF || testIC.scl == 0xFF) {
         PcPort.sendBadI2CPins();
         return;
      }

      // GPIO-Werte setzen (<- packet)
      testIC.setGpioData();

      // I2C-Datenpakete an Test-IC senden
      // ----------------------------------
      // index auf Index des Bytes mit der Anzahl der Xmit-Pakete
      uint8_t index = testIC.numberOfPins;

      uint8_t packetCount = packet.data[index++];
      int rtc = SoftI2cEx.begin(softSDA, softSCL);
      if (rtc != 0) { // Fehler
         PcPort.sendi2cBeginError();
         return;
      }

      // ber alle Pakete
      // index zeigt auf das erste Byte des Pakets = Anzahl Bytes inkl. Slave-Adresse
      for (uint8_t i = 0; i < packetCount; i++) { // Letztes Paket soll eingelesen werden
         uint8_t byteCount = packet.data[index++];
         // index zeigt auf das erste zu versendende Byte des Pakets == Adresse des Slaves
         rtc = SoftI2cEx.writePacket(packet.data[index], &(packet.data[index + 1]), byteCount - 1);

         if (rtc == UrsArduinoSoftI2cExClass::ErrorCode::NoSlave) {
            PcPort.sendI2cInvalidAdress();
            return;
         }
         if (rtc != byteCount - 1) {
            PcPort.sendI2cIoError();
            return;
         }
         index += byteCount;
         // index auf Anzahl der Bytes des Folgepakets bzw. Gerteadresse zum Einlesen
      }

      // I2C-Datenpaket vom Test-IC einlesen und senden
      // -----------------------
      uint8_t devadr = packet.data[index++];
      uint8_t cnt = packet.data[index];
      rtc = SoftI2cEx.readPacket(devadr, &packet.data[0], cnt);

      packet.type = 12;
      packet.count = cnt + 4;
      // Eingelesene Daten (packet) bertragen
      PcPort.sendResultPacket();

      // GPIO-Werte einlesen (-> packet)
      delay(100);
      testIC.buildGpioDataPacket();
      // Eingelesene Daten (packet) bertragen
      PcPort.sendResultPacket();

      return;
   } // Ende (packet.type == 4)

   PcPort.sendBadPacketType();
}