﻿using System;

namespace IC_DB
{
   /// <summary>
   /// Dient als Basisklasse für spezifische Ausnahmen des IC-Tester-Projekts.
   /// </summary>
   public class IcTesterException : ApplicationException
   {
      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="IcTesterException"/>-Klasse mit einer angegebenen Fehlermeldung.
      /// </summary>
      /// <param name="message">Eine Nachricht, die den Fehler beschreibt.</param>
      public IcTesterException(String message = "") : base(message) { }

      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="IcTesterException"/>-Klasse mit einer angegebenen Fehlermeldung und 
      /// einem Verweis auf die innere Exception, die diese Exception verursacht hat.
      /// </summary>
      /// <param name="message">Eine Nachricht, die den Fehler beschreibt.</param>
      /// <param name="innerException">Die Ausnahme, die die Ursache der aktuellen Ausnahme ist.</param>
      public IcTesterException(String message, Exception innerException) : base(message, innerException) { }
   }

   /// <summary>
   /// Beim Einfügen einer neuen IC-Definition in die Datenbank oder bei der Änderung einer bestehenden wird festgestellt, 
   /// dass ein IC mit dem gleichen Namen schon vorhanden ist.
   /// </summary>
   public class DuplicateIcDefinitionException : IcTesterException
   {
      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="DuplicateIcDefinitionException"/>-Klasse.
      /// </summary>
      /// <param name="name">Die Bezeichnung des vorhanden IC.</param>
      public DuplicateIcDefinitionException(String name) : base(name) { }
   }
   /// <summary>
   /// Beim Ändern oder Löschen wird die IC-Definition nicht wieder gefunden.
   /// </summary>
   /// <remarks>Dieser Fehler sollte eigentlich nie auftreten.</remarks>
   public class IcDefinitionNotFoundException : IcTesterException
   {
      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="DuplicateIcDefinitionException"/>-Klasse.
      /// </summary>
      /// <param name="name">Die Bezeichnung des vorhanden IC.</param>
      public IcDefinitionNotFoundException(String name) : base(name) { }
   }
}