﻿
using System.Reflection;

namespace IC_DB
{
   /// <summary>
   /// Attribut-Klasse zur Hinterlegung logischer Werte.
   /// </summary>
   [AttributeUsage(AttributeTargets.Field)]
   public class LogicLevelAttribute : Attribute
   {
      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="LogicLevelAttribute"/>-Klasse.
      /// </summary>
      /// <param name="logicLevel">Der festzulegende logische Wert.</param>
      public LogicLevelAttribute(LogicLevel logicLevel)
      {
         LogicLevelValue = logicLevel;
      }

      /// <summary>
      /// Ruft den in diesem Attribut gespeicherten logischen Wert ab.
      /// </summary>
      public virtual LogicLevel LogicLevel => LogicLevelValue;

      /// <summary>
      /// Lese-/Schreibeigenschaft, die den gespeicherten logischen Wert direkt ändert.
      /// </summary>
      protected LogicLevel LogicLevelValue { get; set; }

      /// <summary>
      /// Liest das <see cref="LogicLevelAttribute"/> des Enumerationswertes aus.
      /// </summary>
      /// <param name="value">Der Enumerationswerte, dessen <see cref="LogicLevelAttribute"/> ermittelt werden soll.</param>
      /// <returns>Der beim Enumerationswert hintelegte logische Wert.</returns>
      /// <exception cref="ArgumentException">Beim angegebenen Enumerationswert ist kein
      /// <see cref="LogicLevelAttribute"/> hinterlegt.</exception>
      /// <typeparam name="T">Formaler Typ-Parameter</typeparam>
      public static LogicLevel GetLogicLevel<T>(T value) where T : Enum
      {
         FieldInfo? fi = value.GetType().GetField(value.ToString());

         if (fi != null)
         {
            LogicLevelAttribute[] attributes = (LogicLevelAttribute[])fi.GetCustomAttributes(typeof(LogicLevelAttribute), false);
            if (attributes != null && attributes.Length > 0)
            {
               return attributes[0].LogicLevel;
            }
         }
         throw new ArgumentException(value.ToString());
      }
   }
}
