﻿// https://www.codeproject.com/articles/34037/datagridvewtextboxcell-with-span-behaviour

namespace IC_Tester_Controls
{
   /// <summary>
   /// Klasse mit Methoden zur Bearbeitung eines <see cref="DataGridView"/>-Objekts.
   /// </summary>
    static class DataGridViewHelper
    {
        public static bool SingleHorizontalBorderAdded(DataGridView dataGridView)
        {
            return !dataGridView.ColumnHeadersVisible &&
                (dataGridView.AdvancedCellBorderStyle.All == DataGridViewAdvancedCellBorderStyle.Single ||
                 dataGridView.CellBorderStyle == DataGridViewCellBorderStyle.SingleHorizontal);
        }

        public static bool SingleVerticalBorderAdded(DataGridView dataGridView)
        {
            return !dataGridView.RowHeadersVisible &&
                (dataGridView.AdvancedCellBorderStyle.All == DataGridViewAdvancedCellBorderStyle.Single ||
                 dataGridView.CellBorderStyle == DataGridViewCellBorderStyle.SingleVertical);
        }

      /// <summary>
      /// Liefert den Typ einer Zeile.
      /// </summary>
      /// <param name="row">Die zu untersuchende Zeile.</param>
      /// <returns>Der Typ der Zeile.</returns>
      /// <exception cref="NullReferenceException"></exception>
      public static RowType getRowType(this DataGridViewRow row)
      {
         var tag = row.Tag;
         if (tag == null)
            throw new NullReferenceException("RowType");

         return (RowType)tag;
      }
   }

   /// <summary>Typ der DataGrid-Zeile.</summary>
   internal enum RowType // Zur Kennzeichnung der DataGrid-Zeilen.
   {
      /// <summary>Zeile enthält logische Werte für die Pins.</summary>
      IoDef,
      /// <summary>Zeile enthält I²C-Transmit-Daten.</summary>
      I2cXmit
   }
}